/******************************************************************
 * PalmOS includes.
 ******************************************************************/
#include <Common.h>
#include <System/SysAll.h>
#include <System/SysEvtMgr.h>
#include <UI/UIAll.h>

/******************************************************************
 * Application includes.
 ******************************************************************/
#include "callback.h"
#include "resource.h"

#include "Timesheet.h"
#include "Globals.h"

#include "Category.h"
#include "Database.h"
#include "DetailForm.h"
#include "EditCatForm.h"
#include "EditCatEntryForm.h"

/******************************************************************
 * Close the EditCat form, releasing any dynamically allocated memory
 ******************************************************************/
void
EditCatFormClose(FormPtr formPtr)
{
  ListPtr listPtr;

  listPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listID_tsEdit));
  /* this is an extremely dodgy bit of coding that matches the other dodgy bit of coding
   * where we initialise the list with the second (not the first) list pointer */
  MemPtrFree(listPtr->itemsText - 1);
}

/******************************************************************
 * Event handler for the Edit List form.
 ******************************************************************/
Boolean
EditCatFormEventHandler(EventPtr event)
{
  Word result;
  Word selected;
  FormPtr formPtr;
  ControlPtr controlPtr;
  ControlPtr controlPtr2;
  Boolean handled = false;

  if (event->eType == ctlSelectEvent)
    {
      if (event->data.ctlEnter.controlID == buttonID_tsOk)
        {
          /* return to the previous form and redraw the form */
          EditCatFormClose(FrmGetActiveForm());
          FrmReturnToForm(formID_tsDetail);
          /* update all the popup triggers as well (as underlying list contents may have changed */
          formPtr = FrmGetActiveForm();
          /* client popup */
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listID_tsClient));
          selected = LstGetSelection((ListPtr) controlPtr);
          controlPtr2 = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listpopupID_tsClient));
          CtlSetLabel(controlPtr2, LstGetSelectionText((ListPtr) controlPtr, selected));
          /* project popup */
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listID_tsProject));
          selected = LstGetSelection((ListPtr) controlPtr);
          controlPtr2 = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listpopupID_tsProject));
          CtlSetLabel(controlPtr2, LstGetSelectionText((ListPtr) controlPtr, selected));
          /* task popup */
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listID_tsTask));
          selected = LstGetSelection((ListPtr) controlPtr);
          controlPtr2 = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listpopupID_tsTask));
          CtlSetLabel(controlPtr2, LstGetSelectionText((ListPtr) controlPtr, selected));
          /* redraw the form */
          FrmDrawForm(formPtr);
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsNew)
        {
          /* user wants to add a new entry, check if user can add another entry */
          if (TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1] == TSMaxCatEntries)
            {
              /* category is already full */
              FrmAlert(alertID_tsCatFull);
              handled = true;
              return handled;
            }
          /* we can add another entry to the category list */
          TSEditCatEntryIdx = 0;
          FrmPopupForm(formID_tsEditCatEntry);
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsRename)
        {
          /* user wants to rename existing entry */
          formPtr = FrmGetActiveForm();
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listID_tsEdit));
          if ((result = LstGetSelection((ListPtr) controlPtr)) == -1)
            {
              /* user doesn't have a category selected */
              FrmAlert(alertID_tsSelectCat);
              handled = true;
              return handled;
            }
          /* we can renmae an existing entry in the category list */
          TSEditCatEntryIdx = result + 1;       /* add one as entry 0 'none' is not user-editable */
          FrmPopupForm(formID_tsEditCatEntry);
          handled = true;
        }
      else if (event->data.ctlEnter.controlID == buttonID_tsDelete)
        {
          /* user wants to delete existing selected entry */
          formPtr = FrmGetActiveForm();
          controlPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listID_tsEdit));
          if ((result = LstGetSelection((ListPtr) controlPtr)) == -1)
            {
              /* user doesn't have a category selected */
              FrmAlert(alertID_tsSelectCat);
              handled = true;
              return handled;
            }
          /* warn the user and get confirmation of delete */
          if (FrmAlert(alertID_tsConfirmCatDel) == 0)
            {
              /* delete the category user has selected */
              EditCatEntryDeleteEntry(result + 1);      /* add one as 0-'none' entry is not editable or shown to user */
              /* re-initialise this form now that there's one less entry */
              EditCatFormInit(-1, true);
              FrmDrawForm(FrmGetActiveForm());
              /* need to update appropriate list on the 1-down  Detail form */
              if (TSEditCatRecIdx == 1)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListDelete(listID_tsClient, listpopupID_tsClient, 1, TSAppPrefs.numCatEntries[0],
                                            result + 1);
                }
              else if (TSEditCatRecIdx == 2)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListDelete(listID_tsProject, listpopupID_tsProject, 2, TSAppPrefs.numCatEntries[1],
                                            result + 1);
                }
              else if (TSEditCatRecIdx == 3)
                {
                  /* refill the list and update the current selection */
                  DetailFormPopupListDelete(listID_tsTask, listpopupID_tsTask, 3, TSAppPrefs.numCatEntries[2], result + 1);
                }
            }
          handled = true;
        }
    }
  else if (event->eType == frmOpenEvent)
    {
      /* the EditCat form has been opened */
      EditCatFormInit(-1, false);
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
    }

  return handled;
}

/******************************************************************
 * Initialise the edit list form, which is about to be shown.
 * The single parameter specifies which list entry should be
 * selected after initialisation has completed. Pass -1 for no
 * selection. If freeExisting is set, the existing list entry memory
 * is released before the list is re-built.
 ******************************************************************/
void
EditCatFormInit(Int selection, Boolean freeExisting)
{
  CharPtr *catPtr;
  ListPtr listPtr;
  VoidHand listHand;
  FormPtr formPtr;
  TSCatRecType *listRec;

  /* open the resource record user wants to edit */
  formPtr = FrmGetActiveForm();
  listPtr = FrmGetObjectPtr(formPtr, FrmGetObjectIndex(formPtr, listID_tsEdit));
  listHand = DmQueryRecord(TSDatabase, TSEditCatRecIdx);
  listRec = MemHandleLock(listHand);
  /* free previously allocated list entries */
  if (freeExisting)
    {
      /* this is an extremely dodgy bit of coding that matches the other dodgy bit of coding
       * where we initialise the list with the second (not the first) list pointer, see below */
      MemPtrFree(listPtr->itemsText - 1);
    }
  /* rebuild the pointer array in case OS has moved records around in memory */
  catPtr = CatBuildPtrArray(listRec, TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1]);
  /* fill it with the supplied entries */
  LstSetListChoices(listPtr, &(catPtr[1]), TSAppPrefs.numCatEntries[TSEditCatRecIdx - 1] - 2);
  MemHandleUnlock(listHand);
  /* set the list's selection */
  LstSetSelection(listPtr, selection);
}

/*** End of file ***/